#include <stdint.h>
#include <stddef.h>
#include "IDHandler.h"


char* SerialNumber::s_SerialNumber = new char[UniqueIDsize];
bool SerialNumber::initCalled = false;

void SerialNumber::init() {
  if (initCalled == true) return;
  for(size_t i = 0; i < UniqueIDsize; i++) {
    s_SerialNumber[i] = (char)UniqueID[i];
  }
  initCalled = true;
}
int SerialNumber::getSNSize() {
  if (initCalled == false) init();
  return UniqueIDsize;
}
char SerialNumber::getSNChar(int p_Index) {
  if (initCalled == false) init();
  if (p_Index < 0) return 0xFF;
  if (p_Index >= UniqueIDsize) return 0xFF;
  return s_SerialNumber[p_Index];
}

uint16_t SerialNumber::getCheckSum_16() {
  if (initCalled == false) init();
  uint32_t c0, c1;
  size_t len;
	size_t blocklen = UniqueIDsize;
  char* data;
 
  len = UniqueIDsize;
  data = s_SerialNumber;
	/*  Found by solving for c1 overflow: */
	/* n > 0 and n * (n+1) / 2 * (2^8-1) < (2^32-1). */
	for (c0 = c1 = 0; len > 0; ) {
    blocklen = len;
		if (blocklen > 5802) {
			blocklen = 5802;
		}
		len -= blocklen;
		do {
			c0 = c0 + *data++;
			c1 = c1 + c0;
		} while (--blocklen);
		c0 = c0 % 255;
		c1 = c1 % 255;
   }
   return (c1 << 8 | c0);
}

uint32_t SerialNumber::getCheckSum_32() {
  if (initCalled == false) init();
  char* data;
  size_t len;

  data = s_SerialNumber;
  len = UniqueIDsize;
	uint32_t c0, c1;
	len = (len + 1) & ~1;      /* Round up len to words */

	/* We similarly solve for n > 0 and n * (n+1) / 2 * (2^16-1) < (2^32-1) here. */
	/* On modern computers, using a 64-bit c0/c1 could allow a group size of 23726746. */
	for (c0 = c1 = 0; len > 0; ) {
		size_t blocklen = len;
		if (blocklen > 360*2) {
			blocklen = 360*2;
		}
		len -= blocklen;
		do {
			c0 = c0 + *data++;
			c1 = c1 + c0;
		} while ((blocklen -= 2));
		c0 = c0 % 65535;
		c1 = c1 % 65535;
	}
	return (c1 << 16 | c0);
}

