#include "Action.h"


//************************** Action Base Class*******************
//--- static ---

List<Action>* Action::s_List_Actions = new List<Action>(30);

bool Action::addAction(Action *p_Action) {
  return s_List_Actions->addElement(p_Action);
}
bool Action::removeAction(Action *p_Action) {
  return s_List_Actions->removeElement(p_Action);
}
int Action::getActionCount() {
  return s_List_Actions->getSize();
}
Action* Action::getAction(int p_Index) {
  return s_List_Actions->getElement(p_Index);
}
Action* Action::deleteAction(Action *p_Action) {
  if (p_Action == NULL) return NULL;
  delete p_Action;
  return NULL;
}

//--- non static ---

Action::Action() {
//  m_ActionTimer = new ActionTimer();
  addAction(this);
  TimerHandler::addTimerListener(this);

}
Action::~Action() {
  removeAction(this);
  TimerHandler::removeTimerListener(this);
}




//************************** Action_Test *******************
Action_Test::Action_Test(String p_String) : Action() {
  m_String = p_String;
}
bool Action_Test::run(double p_Value_In) {
  Serial.print(m_String);
  Serial.print(" Value: ");
  Serial.println(p_Value_In);
  
  return true;
}
void Action_Test::eventTimer(uint64_t p_ActualMicros) {

}

//************************** Action_Blink *******************
Action_Blink::Action_Blink(Action_Output* p_Output, unsigned long p_TimeHigh, unsigned long p_TimeLow ,int p_Repeats, int p_Mode) : Action() {
  m_Output = p_Output;
  m_TimeHigh = p_TimeHigh;
  m_TimeLow = p_TimeLow;
  m_Repeats = p_Repeats;
  m_Mode = p_Mode;
}
Action_Blink::~Action_Blink() {

}
bool Action_Blink::run(double p_Value_In) {
  if (m_Output == NULL) return false;
  if (m_TimeHigh <= 0) return false;
  if (m_TimeLow < 0) m_TimeLow = 0;
  if (m_Repeats == 0) return false;

  switch (m_Mode) {
    case MODE_START_INLOW_STOP_INHIGH:
      if (p_Value_In == LOW) start();
      else stop();
      break;
    case MODE_STARTSTOP_INLOW:
      if (p_Value_In == LOW) {
        if (m_Started == false) start();
        else stop();
      }
      break;
    case MODE_STARTSTOP_INHIGH:
      if (p_Value_In == HIGH) {
        if (m_Started == false) start();
        else stop();
      }
      break;
    case MODE_RUNONCE_INLOW:
      if (p_Value_In == LOW) {
        runOnce();
      }
      break;
    case MODE_RUNONCE_INHIGH:
      if (p_Value_In == HIGH) {
        runOnce();
      }
      break;
    case MODE_RUNONCE_INHIGHLOW:
      runOnce();
      break;
  }  
  return true;
}

void Action_Blink::start() {
  if (m_Started == true) return;
  m_ShouldRun = true;
  m_ActualRepeats = m_Repeats;
}
void Action_Blink::stop() {
  if (m_Started == false) return;
  m_ShouldRun = false;
  m_ActualRepeats = 0;
  eventTimer(TimerHandler::getTime_ActualMicros());
}
void Action_Blink::runOnce() {
  if (m_Repeats < 0) {
    //don't run sequence infinite
    return;
  }
  else {
    m_ActualRepeats += m_Repeats; //increase/set m_ActualRepeats by/to m_Repeats to get a response on every input event
  }
  m_ShouldRun = true;
}

void Action_Blink::eventTimer(uint64_t p_ActualMicros) {
  if (m_ShouldRun == false) {
    //blinking sequence should not run
    if (m_Started == true) {
      if (m_Output != NULL) {
        m_Output->writeValue(LOW);
      }
      m_Started = false;
    }
    return;
  }
  uint64_t uint64TimeDiff;

  if (m_Started == false) {
    //start blinking
    m_ActualDelay = m_TimeHigh;
    m_OutputValue = HIGH;
    m_Started = true;
    m_LastSwapMillis = TimerHandler::getTime_ActualMillis();
  }
  else {
      uint64TimeDiff = TimerHandler::getTimeDiff_Millis(m_LastSwapMillis);
      //instead of using delay, we check the elapsed time - > this function returns withot blocking like  delay() does.
      if (m_Started == true && uint64TimeDiff < m_ActualDelay) {
        return;
      }
      m_LastSwapMillis = TimerHandler::getTime_ActualMillis();

      if (m_OutputValue == HIGH) {
        //swap
        m_OutputValue = LOW;
        m_ActualDelay = m_TimeLow;
        //one blink seuqence done -> reduce repeats
        if (m_ActualRepeats > 0) {
          //_Repeats > 0 -> m_m_ActualRepeats is decreased after each sequence
          m_ActualRepeats--;
        }
      }
      else {
        //m_OutputValue == LOW
        //swap
        m_ActualDelay = m_TimeHigh;
        m_OutputValue = HIGH;
      }
  }

  if (m_Output != NULL) {
    m_Output->writeValue(m_OutputValue);
  }

  if (m_ActualRepeats == 0) {
    //reset the flags
    m_ShouldRun = false;
    m_ActualRepeats = 0;
    m_LastSwapMillis = 0;
  }
}

//************************** Action_HID *******************
Action_HID::Action_HID() {
  m_HIDMouseKeyboard = new HIDMouseKeyboard();
}
Action_HID::Action_HID(uint8_t p_Key, uint8_t p_MouseButton) : Action() {
  Action_HID();
  m_Key = p_Key;
  m_MouseButton = p_MouseButton;

}
void Action_HID::setKeyboardMouseEvent(uint8_t p_Key, uint8_t p_MouseButton) {
  m_MouseButton = p_MouseButton;
  m_Key = p_Key;
}
bool Action_HID::run(double p_Value_In) {
  if (m_Key != 0) {
    if (p_Value_In <= 0) {
      m_HIDMouseKeyboard->keyPreassed(m_Key);
    }
    else {
      m_HIDMouseKeyboard->keyReleased();
    }
  }
  if (m_MouseButton != 0) {
    if (p_Value_In <= 0) {
      m_HIDMouseKeyboard->mousePressed(m_MouseButton);
    }
    else {
      m_HIDMouseKeyboard->mouseReleased();
    }
    
  }
  return true;
 }
 void Action_HID::eventTimer(uint64_t p_ActualMicros) {

 }


//************************** Action_Serial *******************
Action_Serial::Action_Serial() : Action() {

}
Action_Serial::~Action_Serial() {

}

void Action_Serial::setAction_Serial_Interface(Action_Serial_InputInterface* p_Action_Serial_Input) {
  m_Action_Serial_Input = p_Action_Serial_Input;
}
void Action_Serial::setMessage(char* p_MessageBuffer, size_t p_MessageLength) {
  if (p_MessageLength > MESSAGEBUFFER_LENGTH) {
    p_MessageLength = MESSAGEBUFFER_LENGTH;
  }
  m_MessageLength = p_MessageLength;
  for (size_t intByte = 0; intByte < p_MessageLength; intByte++) {
    m_MessageBuffer[intByte] = p_MessageBuffer[intByte];
  }
}
void Action_Serial::setMessage(String p_MessageString) {
  if (p_MessageString == NULL) {
    m_MessageLength = 0;
    return;
  }
  int intLength = p_MessageString.length() + 1; //+1 fr terminator 0x00
  if (intLength > MESSAGEBUFFER_LENGTH) {
    intLength = MESSAGEBUFFER_LENGTH;
  }

  p_MessageString.toCharArray(m_MessageBuffer, intLength);
  p_MessageString[intLength-1] = 0x00;
  m_MessageLength = intLength-1;

}
bool Action_Serial::run(double p_Value_In) {
//  if (!Serial) {
//    return false;
//  }
  if (p_Value_In == LOW) {
    //an Action_serial_Interface is set -> get the message from there
    if (m_Action_Serial_Input != NULL) {
      if (m_Action_Serial_Input->getMessageSerial_Length() <= 0) {
        return false;
      }
      Serial.write(m_Action_Serial_Input->getMessageSerial_Buffer(), m_Action_Serial_Input->getMessageSerial_Length());
      Serial.flush();
    }
    
    else {
      //no Action_serial_Interface is set -> use local message
      if (m_MessageLength <= 0) {
        return false;
      }
      Serial.write(m_MessageBuffer, m_MessageLength);
    }
  }
  return true;
}
void Action_Serial::eventTimer(uint64_t p_ActualMicros) {

}

