#include "TimerHandler.h"
//--- static ---
List<TimerListener>* TimerHandler::s_List_TimerListeners = new List<TimerListener>(50);
unsigned long TimerHandler::s_LastMicros = 0;
unsigned long TimerHandler::s_TimerOverflows = 0;
uint64_t TimerHandler::s_ActualTime_Micros = 0;

/** adds a TimerListener instance.
*/ 
bool TimerHandler::addTimerListener(TimerListener *p_TimerListener) {
  return s_List_TimerListeners->addElement(p_TimerListener);
}
/** Removes a TimerListener instance
*/
bool TimerHandler::removeTimerListener(TimerListener *p_TimerListener) {
  return s_List_TimerListeners->removeElement(p_TimerListener);
}
int TimerHandler::getTimerListenerCount() {
  return s_List_TimerListeners->getSize();
}
TimerListener* TimerHandler::getTimerListener(int p_Index) {
  return s_List_TimerListeners->getElement(p_Index);
}

/** Refreshes the actual time (micros).
* If the time is changed all TimerListeners will be notified by calling there eventTime methode.
*/
void TimerHandler::refreshTime() {
  unsigned long longActualMicros;
  TimerListener* ptrTimerListener;


  longActualMicros = micros();

  if (longActualMicros != s_LastMicros) {
    if (longActualMicros < s_LastMicros) {
      //Timer overflow  happens every 4295 Seconds
      s_TimerOverflows++;
    }
    s_LastMicros = longActualMicros;

    s_ActualTime_Micros = s_TimerOverflows * ULONG_LONG_MAX + longActualMicros;

    for (int intListener = 0; intListener < getTimerListenerCount(); intListener++) {
      ptrTimerListener = getTimerListener(intListener);
      if (ptrTimerListener == NULL) continue;
      ptrTimerListener->eventTimer(s_ActualTime_Micros);
    }
  }
 }
uint64_t TimerHandler::getTime_ActualMicros() {
  return s_ActualTime_Micros;
}
uint64_t TimerHandler::getTimeDiff_Micros(uint64_t p_TimeMicros) {
  return s_ActualTime_Micros - p_TimeMicros;
}
uint64_t TimerHandler::getTime_ActualMillis() {
  return s_ActualTime_Micros / 1000;
}
uint64_t TimerHandler::getTimeDiff_Millis(uint64_t p_TimeMillis) {
  uint64_t uint64TimeActualMillis;
  uint64TimeActualMillis = s_ActualTime_Micros / 1000;
  return uint64TimeActualMillis - p_TimeMillis;
}
