#include <Arduino.h>
#include "InOutputHandler.h"
#include "LoraHandler.h"
#include "IDHandler.h"


// #define Pin_Output_Vibration 12
#define Pin_Output_LED 13
#define Pin_Output_Vibrate 12


#define Pin_Input_Button_A 18
#define Pin_Input_Button_B 5
#define Pin_Input_Button_C 17
#define Pin_Input_Button_D 6
#define Pin_Input_Button_E 16
#define Pin_Input_Button_F 10
#define Pin_Input_Button_G 15
#define Pin_Input_Button_H 11

#define Pin_SPI_CS 19
#define Pin_SPI_SCK 24
#define Pin_SPI_MOSI 23
#define Pin_SPI_MISO 22

#define VBATPIN A7 //== Pin 9 //Adafruit feather mo battery analog input



//--------------------- MAIN --------------------
uint32_t  uintBoardID; //checksum of the serial number of this arduino hardware

Output_Digital* o_Output_LED;
Output_Digital* o_Output_Vibrate;

Action_Blink* o_Action_Blink_Start;
Action_Blink* o_Action_Blink_StartVibrate;


Input_Digital* o_Input_Digital_A;
Action_Lora_Send* o_Action_Lora_A;
Input_Digital* o_Input_Digital_B;
Action_Lora_Send* o_Action_Lora_B;
Input_Digital* o_Input_Digital_C;
Action_Lora_Send* o_Action_Lora_C;
Input_Digital* o_Input_Digital_D;
Action_Lora_Send* o_Action_Lora_D;
Input_Digital* o_Input_Digital_E;
Action_Lora_Send* o_Action_Lora_E;
Input_Digital* o_Input_Digital_F;
Action_Lora_Send* o_Action_Lora_F;
Input_Digital* o_Input_Digital_G;
Action_Lora_Send* o_Action_Lora_G;
Input_Digital* o_Input_Digital_H;
Action_Lora_Send* o_Action_Lora_H;

Input_Lora* o_Input_Lora;
Action_Blink* o_Action_Blink_LED;
Action_Blink* o_Action_Blink_Vibrate;

bool checkBatteryOnce();

void setup() {
  
  // put your setup code here, to run once:
  
  uintBoardID = SerialNumber::getCheckSum_32(); //we use the checksum of the board's serial number as Lora SenderID (4 byte)
  
  //--- Outputs ---
  o_Output_LED = new Output_Digital(Pin_Output_LED);
  o_Output_Vibrate = new Output_Digital(Pin_Output_Vibrate);

  //Initialize Lora
  if (Lora::init(o_Output_LED) == true) {
    //Lora successfull initialized
    //SENDER add the checksum of the serial number of this ArduinoHardware
    if (Lora::addSenderID(uintBoardID) ==true) {
      //Sender ID added
    }
  }
  else {
    //error Lora initialization
  }
  


    //--- Lora ---
  o_Input_Lora = new Input_Lora();
  o_Action_Blink_LED = new Action_Blink(o_Output_LED, 200, 50, 1, Action_Blink::MODE_RUNONCE_INLOW);
  o_Action_Blink_Vibrate = new Action_Blink(o_Output_Vibrate, 200, 50, 1, Action_Blink::MODE_RUNONCE_INLOW);
  o_Input_Lora->addAction(o_Action_Blink_LED);
  o_Input_Lora->addAction(o_Action_Blink_Vibrate);

  
  //Buttons
  o_Input_Digital_A = new Input_Digital(Pin_Input_Button_A);
  o_Action_Lora_A = new Action_Lora_Send(uintBoardID, '0', String((char)(Lora::DEFAULT_STARTCODE + 0))); //sending lora message
  o_Input_Digital_A->addAction(o_Action_Lora_A);
  o_Input_Digital_A->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession 

  o_Input_Digital_B = new Input_Digital(Pin_Input_Button_B);
  o_Action_Lora_B = new Action_Lora_Send(uintBoardID, '0', String((char)(Lora::DEFAULT_STARTCODE + 1))); //sending lora message
  o_Input_Digital_B->addAction(o_Action_Lora_B);
  o_Input_Digital_B->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession 

  o_Input_Digital_C = new Input_Digital(Pin_Input_Button_C);
  o_Action_Lora_C = new Action_Lora_Send(uintBoardID, '0', String((char)(Lora::DEFAULT_STARTCODE + 2))); //sending lora message
  o_Input_Digital_C->addAction(o_Action_Lora_C);
  o_Input_Digital_C->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession  

  o_Input_Digital_D = new Input_Digital(Pin_Input_Button_D);
  o_Action_Lora_D = new Action_Lora_Send(uintBoardID, '0', String((char)(Lora::DEFAULT_STARTCODE + 3))); //sending lora message
  o_Input_Digital_D->addAction(o_Action_Lora_D);
  o_Input_Digital_D->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession 

  o_Input_Digital_E = new Input_Digital(Pin_Input_Button_E);
  o_Action_Lora_E = new Action_Lora_Send(uintBoardID, '0', String((char)(Lora::DEFAULT_STARTCODE + 4))); //sending lora message
  o_Input_Digital_E->addAction(o_Action_Lora_E);
  o_Input_Digital_E->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession 

  o_Input_Digital_F = new Input_Digital(Pin_Input_Button_F);
  o_Action_Lora_F = new Action_Lora_Send(uintBoardID, '0', String(char(Lora::DEFAULT_STARTCODE + 5))); //sending lora message
  o_Input_Digital_F->addAction(o_Action_Lora_F);
  o_Input_Digital_F->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession 

  o_Input_Digital_G = new Input_Digital(Pin_Input_Button_G);
  o_Action_Lora_G = new Action_Lora_Send(uintBoardID, '0', String((char)(Lora::DEFAULT_STARTCODE + 6))); //sending lora message
  o_Input_Digital_G->addAction(o_Action_Lora_G);
  o_Input_Digital_G->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession 

  o_Input_Digital_H = new Input_Digital(Pin_Input_Button_H);
  o_Action_Lora_H = new Action_Lora_Send(uintBoardID, '0', String((char)(Lora::DEFAULT_STARTCODE + 7))); //sending lora message
  o_Input_Digital_H->addAction(o_Action_Lora_H);
  o_Input_Digital_H->setDebounceTimeSpan(250); //Set long debounce timespan->don`t send loramessage in quick succession 

  bool boolBatteryOK = true;
  boolBatteryOK = checkBatteryOnce();
  if (boolBatteryOK == true) {
    //if battery is ok -> blink at start
    o_Action_Blink_Start = new Action_Blink(o_Output_LED, 500,500,3,Action_Blink::MODE_RUNONCE_INLOW);
    o_Action_Blink_StartVibrate = new Action_Blink(o_Output_Vibrate, 500,500,3,Action_Blink::MODE_RUNONCE_INLOW);
    o_Action_Blink_Start->run(LOW);
    o_Action_Blink_StartVibrate->run(LOW);
  }
}

void loop() {
 
  //AVOID TO USE dley OR sleep in the loop
  //INSTEAD try to use the TimerHandler

  //Refresh TimerHandler, all TimerListeners will be notified
  TimerHandler::refreshTime(); //this is important for Actions and Input debouncing

  Input* ptrInput;

  //go through all input instances
  for (int intInput = 0; intInput < Input::getInputCount(); intInput++) {
    ptrInput = Input::getInput(intInput);
    if (ptrInput == NULL) continue; //should never happen

    if (ptrInput->isInterruptable() == false) {
      //the input has no interrupt -> call checkInput to check whether the value has changed
      ptrInput->checkInput();
    }

    //check if the input is changed
    if (ptrInput->getValueChanged() == true) {
      ptrInput->runActions(); //run actions if the ptrInput has an action assigned
      //if extra actions should start, check if ptrInput is the instance of the class you want and then call the several actions
      //if (ptrInput == m_inputXXX) {
      //  ActionX.run(ValueX); //maybe ValueX = ptrInput->getValue();
      //  ActionY.run(ValueY);
      //}
      ptrInput->processed(); //has to be called when the changed value is handled - this will set Input's m_ValueChanged to false


    }
  }

  
  
  if (Input::getAllInterruptable() == true || Input::getInputCount() == 0) {
    //all inputs are interruptable
    //When all Actions are done
    //-> perhaps we can use deepSleep ???
//    Serial.println("All Inputs interruptable");
  }
 
}

bool checkBatteryOnce() {
  //VBATPIN == PIn 9 
  //Use VBATPIN one time at start 
  float measuredvbat = analogRead(VBATPIN);
  measuredvbat *= 2; // we divided by 2, so multiply back
  measuredvbat *= 3.3; // Multiply by 3.3V, our reference voltage
  measuredvbat /= 1024; // convert to voltage

  //We use the pin 9 == A/ as digital pin -> set to input_pullup
  pinMode(VBATPIN, INPUT_PULLUP);

  if (measuredvbat < 3.6f) {
    //LiPoly batteries are 'maxedout' at 4.2V 
    //and stick around 3.7V for much of the battery life, then slowly sink down to 3.2V or so before the protection circuitry cuts it off.
    //battery charge is low
    return false;
  }
  //battery charge is ok
  return true;
}